import * as i2 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i0 from '@angular/core';
import { Component, ChangeDetectionStrategy, Optional, Self, Input, HostListener } from '@angular/core';
import * as i1 from '@angular/forms';
import { ReactiveFormsModule, FormsModule } from '@angular/forms';
import { debounceTime, distinctUntilChanged } from 'rxjs';

const THEMES = {
    light: {
        color: '#d32f2f',
        fontSize: '12px',
        backgroundColor: '#ffffff',
    },
    dark: {
        color: '#ff5252',
        fontSize: '12px',
        backgroundColor: '#333333',
    },
    compact: {
        fontSize: '10px',
        color: '#d32f2f',
        backgroundColor: '#ffffff',
    },
};

// Default configuration settings for the utility
const defaultConfig = {
    cacheEnabled: true,
    stylePriority: ['theme', 'class', 'custom'],
    errorHandler: (error, styleSheetHref) => {
        console.warn('Cannot access stylesheet:', styleSheetHref, error);
    },
};
// Cache to store extracted styles for class names
const styleCache = {};
/**
 * Merges styles from a specified theme, class, and any custom styles provided.
 *
 * @param themeName - The name of the theme to retrieve styles from (e.g., 'light', 'dark').
 * @param className - The class name(s) applied to the element for additional styling. Overrides theme styles.
 * @param customStyles - An object containing custom inline styles that will override both theme and class styles.
 * @param config - Configuration options to control caching, style source priority, and error handling.
 * @returns An object containing the merged styles with the specified precedence order.
 */
function getMergedStyles(themeName, className, customStyles = {}, config = defaultConfig) {
    // Retrieve base theme styles if no class styles are specified
    const themeStyles = THEMES[themeName] || {};
    // Handle multiple classes if provided, merge extracted styles
    const classStyles = className
        ? mergeMultipleClassStyles(className.split(' '), config)
        : {};
    // Determine the order of merging based on the config priority
    const styles = {
        theme: themeStyles,
        class: classStyles,
        custom: customStyles,
    };
    // Merge styles based on the configured priority
    const mergedStyles = config.stylePriority?.reduce((acc, key) => ({ ...acc, ...styles[key] }), {});
    return mergedStyles || {};
}
/**
 * Merges styles from multiple classes, ensuring later classes override earlier ones.
 * Caches results to improve performance for frequently used classes.
 *
 * @param classNames - An array of class names to extract and merge styles from.
 * @param config - Configuration options that control caching and error handling.
 * @returns An object containing the merged CSS properties and values from all specified classes.
 */
function mergeMultipleClassStyles(classNames, config) {
    return classNames.reduce((acc, className) => {
        // Use caching if enabled
        if (config.cacheEnabled && styleCache[className]) {
            return { ...acc, ...styleCache[className] };
        }
        // Extract styles for the class and cache if enabled
        const extractedStyles = extractStylesFromClass(`.${className}`, config);
        if (config.cacheEnabled) {
            styleCache[className] = extractedStyles;
        }
        return { ...acc, ...extractedStyles };
    }, {});
}
/**
 * Extracts styles from the specified class name by accessing document style sheets.
 * Handles large projects efficiently by caching results and handling errors gracefully.
 *
 * @param className - The class name (e.g., '.error') whose styles you want to extract.
 * @param config - Configuration options that control error handling.
 * @returns An object containing the extracted CSS properties and their values.
 */
function extractStylesFromClass(className, config) {
    const extractedStyles = {};
    const styleSheets = Array.from(document.styleSheets);
    // Iterate through each style sheet
    styleSheets.forEach((styleSheet) => {
        // Some style sheets may not be accessible due to CORS policies; use error handler if set
        try {
            const cssRules = styleSheet.cssRules || styleSheet.rules; // Get CSS rules
            // Iterate through each rule in the style sheet
            Array.from(cssRules).forEach((rule) => {
                // Check if the rule is a style rule and matches the class name
                if (rule instanceof CSSStyleRule &&
                    rule.selectorText.includes(className)) {
                    // Extract styles from the matched rule
                    for (let i = 0; i < rule.style.length; i++) {
                        const propertyName = rule.style[i];
                        const propertyValue = rule.style.getPropertyValue(propertyName);
                        extractedStyles[propertyName] = propertyValue;
                    }
                }
            });
        }
        catch (error) {
            // Use the custom error handler defined in the configuration
            config.errorHandler?.(error, styleSheet.href || '');
        }
    });
    return extractedStyles;
}

/**
 * SmartErrorDisplayComponent
 *
 * A robust Angular component that displays validation errors for a given form control or form group.
 * This component offers advanced theming, localization, custom error formatting, accessibility features,
 * performance optimizations, and analytics integration, making it suitable for production use in any Angular application.
 */
class SmartErrorDisplay {
    ngControl;
    cdr;
    renderer;
    /**
     * The form control or form group whose errors will be displayed.
     * Supports both reactive form controls (AbstractControl) and template-driven form controls (NgControl).
     */
    control = null;
    /**
     * Custom error messages to override default messages.
     * The key represents the error type (e.g., 'required'), and the value is the custom message.
     */
    customMessages = null;
    /**
     * Inline styles to apply to the error message container, allowing for custom theming.
     */
    themeStyles = {};
    /**
     * CSS class to apply to the error message container for additional styling or theming.
     */
    themeClass = null;
    /**
     * Predefined theme option for the error messages ('light', 'dark', or 'compact').
     * Accepted values: 'light', 'dark', 'compact'.
     */
    theme = 'light';
    /**
     * Custom formatter function to format error messages dynamically based on the error key and value.
     */
    errorFormatter = () => null;
    /**
     * Controls when the error messages are displayed: on value change, focus, blur, or hover.
     * Accepted values: 'change', 'focus', 'blur', 'hover'.
     */
    displayOn = 'change';
    /**
     * Configurable debounce time for value changes in milliseconds. Default is 300ms.
     */
    debounceTime = 300;
    /**
     * Optional translation service for localizing error messages.
     * Should implement a `translate()` method that takes a message string and returns the translated version.
     */
    translationService = null;
    /**
     * Optional analytics service for logging errors, useful for tracking user behavior and form issues.
     * Should implement a `logEvent()` method that logs the error event.
     */
    analyticsService = null;
    /**
     * Hook to customize how logging is handled. Allows integration with various analytics services.
     * Defaults to using `analyticsService.logEvent`.
     */
    logError = this.defaultLogError;
    /**
     * Default messages used when custom messages are not provided.
     */
    defaultMessages = {
        required: 'This field is required.',
        minlength: 'The value is too short.',
        maxlength: 'The value is too long.',
        email: 'Please enter a valid email address.',
        pattern: 'The value does not match the required pattern.',
        min: 'The value is too low.',
        max: 'The value is too high.',
    };
    /**
     * Template reference for the associated input element.
     */
    inputRef;
    /**
     * Merged styles applied to the error display container.
     * Combines the styles from the selected theme and any custom styles provided via `themeStyles`.
     * This object uses an index signature to allow flexible CSS property-value pairs.
     */
    mergedStyles = {};
    /**
     * Flag to determine if errors should be displayed based on the control's state and the display trigger.
     */
    shouldDisplayError = false;
    /**
     * Internal state variables to track focus.
     */
    _focused = false;
    /**
     * Internal state variables to track hover.
     */
    _hovered = false;
    /**
     * Subscription management to handle clean up.
     */
    valueChangeSubscription = null;
    constructor(ngControl, cdr, renderer) {
        this.ngControl = ngControl;
        this.cdr = cdr;
        this.renderer = renderer;
    }
    /**
     * Initializes the component and sets up subscriptions to form control value changes.
     * Automatically detects whether the control is provided via reactive forms or template-driven forms.
     */
    ngOnInit() {
        // Use the injected NgControl if control input is not provided (for template-driven forms)
        if (!this.control && this.ngControl) {
            this.control = this.ngControl.control;
        }
        // Validate inputs
        this.validateInputs();
        if (this.control) {
            // Subscribe to value changes with configurable debounce time
            this.valueChangeSubscription = this.control.valueChanges
                .pipe(debounceTime(this.debounceTime), distinctUntilChanged())
                .subscribe({
                next: () => {
                    this.updateDisplayErrorState();
                    this.cdr.detectChanges();
                },
                error: (err) => this.handleComponentError(err),
            });
        }
    }
    /**
     * Lifecycle hook that is called after a component's view has been fully initialized.
     * This method attaches input listeners and merges styles based on the provided theme,
     * theme class, and theme styles.
     */
    ngAfterViewInit() {
        this.attachInputListeners();
        this.mergedStyles = getMergedStyles(this.theme, this.themeClass, this.themeStyles);
    }
    /**
     * Returns a list of formatted error messages based on the control's current errors.
     * It uses custom messages, default messages, or a provided error formatter function.
     */
    get errorMessages() {
        if (!this.control || !this.control.errors) {
            return [];
        }
        const errors = Object.entries(this.control.errors);
        return errors.map(([errorKey, errorValue]) => {
            const controlName = this.control?.parent
                ? this.control.parent.getError(errorKey)
                : undefined;
            const message = (this.errorFormatter &&
                this.errorFormatter(errorKey, errorValue, controlName)) ||
                this.customMessages?.[errorKey] ||
                this.getDefaultMessage(errorKey);
            this.logError(errorKey);
            return message;
        });
    }
    /**
     * Provides default error messages for common validation errors.
     * Supports localization through an optional translation service.
     *
     * @param errorKey - The key representing the type of error (e.g., 'required').
     * @returns The default error message or a translated message if a translation service is provided.
     */
    getDefaultMessage(errorKey) {
        const message = this.defaultMessages[errorKey] || 'Invalid input.';
        return this.translationService
            ? this.translationService.translate(message)
            : message;
    }
    /**
     * Default logging function that uses the provided analytics service if available.
     *
     * @param errorKey - The key representing the type of error (e.g., 'required').
     */
    defaultLogError(errorKey) {
        if (this.analyticsService?.logEvent) {
            this.analyticsService.logEvent('form_error', { error: errorKey });
        }
    }
    /**
     * Attaches input listeners directly to the provided input reference.
     */
    attachInputListeners() {
        if (this.displayOn === 'change') {
            return;
        }
        if (!this.inputRef) {
            console.warn(`No input reference provided for display type ${this.displayOn}. Error display may not work as expected.`);
            return;
        }
        const inputElement = this.inputRef?.nativeElement;
        // Ensure listeners are correctly attached
        this.renderer.listen(inputElement, 'focus', this.handleFocus.bind(this));
        this.renderer.listen(inputElement, 'blur', this.handleBlur.bind(this));
        this.renderer.listen(inputElement, 'mouseenter', this.handleMouseEnter.bind(this));
        this.renderer.listen(inputElement, 'mouseleave', this.handleMouseLeave.bind(this));
    }
    /**
     * Updates the state of whether errors should be displayed based on the control's status and user interactions.
     */
    updateDisplayErrorState() {
        if (!this.control) {
            this.shouldDisplayError = false;
            return;
        }
        // Update error display state based on current interaction mode
        const isInvalidAndTouched = this.control.invalid && this.control.touched;
        switch (this.displayOn) {
            case 'change':
                this.shouldDisplayError = isInvalidAndTouched || this.control.dirty;
                break;
            case 'focus':
                this.shouldDisplayError = this.control.invalid && this._focused;
                break;
            case 'blur':
                this.shouldDisplayError = isInvalidAndTouched && !this._focused;
                break;
            case 'hover':
                this.shouldDisplayError = this.control.invalid && this._hovered;
                break;
            default:
                this.shouldDisplayError = isInvalidAndTouched;
        }
        this.cdr.detectChanges();
    }
    /**
     * Handles the focus event of the error display container.
     * Sets the internal state variable `_focused` to true and updates the display error state.
     */
    handleFocus() {
        this._focused = true;
        this.updateDisplayErrorState();
    }
    /**
     * Handles the blur event of the error display container.
     * Sets the internal state variable `_focused` to false and updates the display error state.
     */
    handleBlur() {
        this._focused = false;
        this.updateDisplayErrorState();
    }
    /**
     * Handles the mouse enter event of the error display container.
     * Sets the internal state variable `_hovered` to true and updates the display error state.
     */
    handleMouseEnter() {
        this._hovered = true;
        this.updateDisplayErrorState();
    }
    /**
     * Handles the mouse leave event of the error display container.
     * Sets the internal state variable `_hovered` to false and updates the display error state.
     */
    handleMouseLeave() {
        this._hovered = false;
        this.updateDisplayErrorState();
    }
    /**
     * Cleanup logic to prevent memory leaks by unsubscribing from value changes.
     */
    ngOnDestroy() {
        this.valueChangeSubscription?.unsubscribe();
    }
    /**
     * Validates inputs to ensure they are within expected ranges and types.
     */
    validateInputs() {
        // Validate theme input
        const validThemes = ['light', 'dark', 'compact'];
        if (!validThemes.includes(this.theme)) {
            console.warn(`Invalid theme '${this.theme}'. Defaulting to 'light'.`);
            this.theme = 'light';
        }
        // Validate displayOn input
        const validDisplayOn = ['change', 'focus', 'blur', 'hover'];
        if (!validDisplayOn.includes(this.displayOn)) {
            console.warn(`Invalid displayOn value '${this.displayOn}'. Defaulting to 'change'.`);
            this.displayOn = 'change';
        }
        // Validate debounceTime input
        if (this.debounceTime < 0) {
            console.warn(`Invalid debounceTime '${this.debounceTime}'. Defaulting to 300ms.`);
            this.debounceTime = 300;
        }
    }
    /**
     * Handles component errors and provides a fallback mechanism.
     *
     * @param error - The error object caught during the execution of the component.
     */
    handleComponentError(error) {
        console.error('Error detected in SmartErrorDisplayComponent:', error);
        // Display generic error message or highlight the affected input field
        this.shouldDisplayError = true;
        this.mergedStyles = { ...this.mergedStyles, border: '1px solid red' };
        this.cdr.markForCheck();
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.6", ngImport: i0, type: SmartErrorDisplay, deps: [{ token: i1.NgControl, optional: true, self: true }, { token: i0.ChangeDetectorRef }, { token: i0.Renderer2 }], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.6", type: SmartErrorDisplay, isStandalone: true, selector: "smart-error-display", inputs: { control: "control", customMessages: "customMessages", themeStyles: "themeStyles", themeClass: "themeClass", theme: "theme", errorFormatter: "errorFormatter", displayOn: "displayOn", debounceTime: "debounceTime", translationService: "translationService", analyticsService: "analyticsService", logError: "logError", defaultMessages: "defaultMessages", inputRef: "inputRef" }, host: { listeners: { "focus": "handleFocus($event.target)", "blur": "handleBlur($event.target)", "mouseenter": "handleMouseEnter($event.target)", "mouseleave": "handleMouseLeave($event.target)" } }, ngImport: i0, template: `
    <div
      *ngIf="shouldDisplayError"
      [ngStyle]="mergedStyles"
      aria-live="assertive"
      role="alert"
      id="smart-error-display"
    >
      <div *ngFor="let error of errorMessages">
        {{ error }}
      </div>
    </div>
  `, isInline: true, dependencies: [{ kind: "ngmodule", type: CommonModule }, { kind: "directive", type: i2.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i2.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i2.NgStyle, selector: "[ngStyle]", inputs: ["ngStyle"] }, { kind: "ngmodule", type: ReactiveFormsModule }, { kind: "ngmodule", type: FormsModule }], changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.6", ngImport: i0, type: SmartErrorDisplay, decorators: [{
            type: Component,
            args: [{
                    selector: 'smart-error-display',
                    standalone: true,
                    imports: [CommonModule, ReactiveFormsModule, FormsModule],
                    template: `
    <div
      *ngIf="shouldDisplayError"
      [ngStyle]="mergedStyles"
      aria-live="assertive"
      role="alert"
      id="smart-error-display"
    >
      <div *ngFor="let error of errorMessages">
        {{ error }}
      </div>
    </div>
  `,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [{ type: i1.NgControl, decorators: [{
                    type: Optional
                }, {
                    type: Self
                }] }, { type: i0.ChangeDetectorRef }, { type: i0.Renderer2 }], propDecorators: { control: [{
                type: Input
            }], customMessages: [{
                type: Input
            }], themeStyles: [{
                type: Input
            }], themeClass: [{
                type: Input
            }], theme: [{
                type: Input
            }], errorFormatter: [{
                type: Input
            }], displayOn: [{
                type: Input
            }], debounceTime: [{
                type: Input
            }], translationService: [{
                type: Input
            }], analyticsService: [{
                type: Input
            }], logError: [{
                type: Input
            }], defaultMessages: [{
                type: Input
            }], inputRef: [{
                type: Input
            }], handleFocus: [{
                type: HostListener,
                args: ['focus', ['$event.target']]
            }], handleBlur: [{
                type: HostListener,
                args: ['blur', ['$event.target']]
            }], handleMouseEnter: [{
                type: HostListener,
                args: ['mouseenter', ['$event.target']]
            }], handleMouseLeave: [{
                type: HostListener,
                args: ['mouseleave', ['$event.target']]
            }] } });

/**
 * Generated bundle index. Do not edit.
 */

export { SmartErrorDisplay };
//# sourceMappingURL=ngx-smart-forms-smart-error-display.mjs.map
