import { AfterViewInit, ChangeDetectorRef, ElementRef, OnDestroy, OnInit, Renderer2 } from '@angular/core';
import { AbstractControl, NgControl } from '@angular/forms';
import * as i0 from "@angular/core";
/**
 * SmartErrorDisplayComponent
 *
 * A robust Angular component that displays validation errors for a given form control or form group.
 * This component offers advanced theming, localization, custom error formatting, accessibility features,
 * performance optimizations, and analytics integration, making it suitable for production use in any Angular application.
 */
export declare class SmartErrorDisplay implements OnInit, AfterViewInit, OnDestroy {
    private ngControl;
    private cdr;
    private renderer;
    /**
     * The form control or form group whose errors will be displayed.
     * Supports both reactive form controls (AbstractControl) and template-driven form controls (NgControl).
     */
    control: AbstractControl | null;
    /**
     * Custom error messages to override default messages.
     * The key represents the error type (e.g., 'required'), and the value is the custom message.
     */
    customMessages: {
        [key: string]: string;
    } | null;
    /**
     * Inline styles to apply to the error message container, allowing for custom theming.
     */
    themeStyles: {
        [key: string]: string;
    };
    /**
     * CSS class to apply to the error message container for additional styling or theming.
     */
    themeClass: string | null;
    /**
     * Predefined theme option for the error messages ('light', 'dark', or 'compact').
     * Accepted values: 'light', 'dark', 'compact'.
     */
    theme: 'light' | 'dark' | 'compact';
    /**
     * Custom formatter function to format error messages dynamically based on the error key and value.
     */
    errorFormatter: (errorKey: string, errorValue: unknown, controlName?: string) => string | null;
    /**
     * Controls when the error messages are displayed: on value change, focus, blur, or hover.
     * Accepted values: 'change', 'focus', 'blur', 'hover'.
     */
    displayOn: 'change' | 'focus' | 'blur' | 'hover';
    /**
     * Configurable debounce time for value changes in milliseconds. Default is 300ms.
     */
    debounceTime: number;
    /**
     * Optional translation service for localizing error messages.
     * Should implement a `translate()` method that takes a message string and returns the translated version.
     */
    translationService: {
        translate: (message: string) => string;
    } | null;
    /**
     * Optional analytics service for logging errors, useful for tracking user behavior and form issues.
     * Should implement a `logEvent()` method that logs the error event.
     */
    analyticsService: {
        logEvent?: (eventName: string, eventData: unknown) => void;
    } | null;
    /**
     * Hook to customize how logging is handled. Allows integration with various analytics services.
     * Defaults to using `analyticsService.logEvent`.
     */
    logError: (errorKey: string, controlName?: string) => void;
    /**
     * Default messages used when custom messages are not provided.
     */
    defaultMessages: {
        [key: string]: string;
    };
    /**
     * Template reference for the associated input element.
     */
    inputRef: ElementRef;
    /**
     * Merged styles applied to the error display container.
     * Combines the styles from the selected theme and any custom styles provided via `themeStyles`.
     * This object uses an index signature to allow flexible CSS property-value pairs.
     */
    mergedStyles: {
        [key: string]: string;
    };
    /**
     * Flag to determine if errors should be displayed based on the control's state and the display trigger.
     */
    shouldDisplayError: boolean;
    /**
     * Internal state variables to track focus.
     */
    private _focused;
    /**
     * Internal state variables to track hover.
     */
    private _hovered;
    /**
     * Subscription management to handle clean up.
     */
    private valueChangeSubscription;
    constructor(ngControl: NgControl, cdr: ChangeDetectorRef, renderer: Renderer2);
    /**
     * Initializes the component and sets up subscriptions to form control value changes.
     * Automatically detects whether the control is provided via reactive forms or template-driven forms.
     */
    ngOnInit(): void;
    /**
     * Lifecycle hook that is called after a component's view has been fully initialized.
     * This method attaches input listeners and merges styles based on the provided theme,
     * theme class, and theme styles.
     */
    ngAfterViewInit(): void;
    /**
     * Returns a list of formatted error messages based on the control's current errors.
     * It uses custom messages, default messages, or a provided error formatter function.
     */
    get errorMessages(): string[];
    /**
     * Provides default error messages for common validation errors.
     * Supports localization through an optional translation service.
     *
     * @param errorKey - The key representing the type of error (e.g., 'required').
     * @returns The default error message or a translated message if a translation service is provided.
     */
    private getDefaultMessage;
    /**
     * Default logging function that uses the provided analytics service if available.
     *
     * @param errorKey - The key representing the type of error (e.g., 'required').
     */
    private defaultLogError;
    /**
     * Attaches input listeners directly to the provided input reference.
     */
    private attachInputListeners;
    /**
     * Updates the state of whether errors should be displayed based on the control's status and user interactions.
     */
    private updateDisplayErrorState;
    /**
     * Handles the focus event of the error display container.
     * Sets the internal state variable `_focused` to true and updates the display error state.
     */
    handleFocus(): void;
    /**
     * Handles the blur event of the error display container.
     * Sets the internal state variable `_focused` to false and updates the display error state.
     */
    handleBlur(): void;
    /**
     * Handles the mouse enter event of the error display container.
     * Sets the internal state variable `_hovered` to true and updates the display error state.
     */
    handleMouseEnter(): void;
    /**
     * Handles the mouse leave event of the error display container.
     * Sets the internal state variable `_hovered` to false and updates the display error state.
     */
    handleMouseLeave(): void;
    /**
     * Cleanup logic to prevent memory leaks by unsubscribing from value changes.
     */
    ngOnDestroy(): void;
    /**
     * Validates inputs to ensure they are within expected ranges and types.
     */
    private validateInputs;
    /**
     * Handles component errors and provides a fallback mechanism.
     *
     * @param error - The error object caught during the execution of the component.
     */
    private handleComponentError;
    static ɵfac: i0.ɵɵFactoryDeclaration<SmartErrorDisplay, [{ optional: true; self: true; }, null, null]>;
    static ɵcmp: i0.ɵɵComponentDeclaration<SmartErrorDisplay, "smart-error-display", never, { "control": { "alias": "control"; "required": false; }; "customMessages": { "alias": "customMessages"; "required": false; }; "themeStyles": { "alias": "themeStyles"; "required": false; }; "themeClass": { "alias": "themeClass"; "required": false; }; "theme": { "alias": "theme"; "required": false; }; "errorFormatter": { "alias": "errorFormatter"; "required": false; }; "displayOn": { "alias": "displayOn"; "required": false; }; "debounceTime": { "alias": "debounceTime"; "required": false; }; "translationService": { "alias": "translationService"; "required": false; }; "analyticsService": { "alias": "analyticsService"; "required": false; }; "logError": { "alias": "logError"; "required": false; }; "defaultMessages": { "alias": "defaultMessages"; "required": false; }; "inputRef": { "alias": "inputRef"; "required": false; }; }, {}, never, never, true, never>;
}
